@@ -24,8 +24,22 @@
 #include "toplevel.h"
 #include "util-gtk.h"
 
+#include "private.h"
+
 G_DEFINE_TYPE(ValaPanelToplevelConfig, vala_panel_toplevel_config, GTK_TYPE_DIALOG);
 
+static GQuark applet_quark;
+#define config_row_get_applet(row)                                                                 \
+	VALA_PANEL_APPLET(g_object_get_qdata((GObject *)row, applet_quark))
+#define config_row_set_applet(row, applet) g_object_set_qdata((GObject *)row, applet_quark, applet)
+
+#define config_row_get_info_data(row)                                                              \
+	((AppletInfoData *)(g_object_get_qdata((GObject *)row, applet_quark)))
+#define config_row_set_info(row, applet) g_object_set_qdata((GObject *)row, applet_quark, applet)
+
+#define NEW_APPLET_NAME "new-applet"
+#define NO_SETTINGS "no-settings"
+
 #define COLUMN_ICON 0
 #define COLUMN_NAME 1
 #define COLUMN_EXPAND 2
@@ -40,9 +54,26 @@ enum
 static GParamSpec *vala_panel_toplevel_config_properties[NUM_PROPERTIES];
 static void state_configure_monitor(GSimpleAction *act, GVariant *param, void *data);
 static void init_plugin_list(ValaPanelToplevelConfig *self);
+static void on_add_plugin_stack_box_generate(ValaPanelToplevelConfig *self);
+static void on_remove_plugin(GtkButton *btn, void *user_data);
+
+void vala_panel_toplevel_config_select_applet(ValaPanelToplevelConfig *self, const char *uuid)
+{
+	GList *ch               = gtk_container_get_children(self->plugin_list);
+	GtkListBoxRow *selected = NULL;
+	for (GList *l = ch; l != NULL; l = g_list_next(l))
+	{
+		ValaPanelApplet *pl = config_row_get_applet(l->data);
+		if (!g_strcmp0(uuid, vala_panel_applet_get_uuid(pl)))
+			selected = GTK_LIST_BOX_ROW(l->data);
+	}
+	gtk_list_box_select_row(self->plugin_list, selected);
+	g_list_free(ch);
+}
 
 static void vala_panel_toplevel_config_init(ValaPanelToplevelConfig *self)
 {
+	applet_quark = g_quark_from_static_string("vala-panel-applet-property");
 	gtk_widget_init_template(GTK_WIDGET(self));
 }
 
@@ -61,9 +92,10 @@ static void vala_panel_configure_dialog_finalize(GObject *obj)
 	g_clear_object(&self->color_foreground);
 	g_clear_object(&self->file_background);
 	g_clear_object(&self->plugin_list);
-	g_clear_object(&self->plugin_desc);
 	g_clear_object(&self->adding_button);
-	g_clear_object(&self->configure_button);
+	g_clear_object(&self->up_button);
+	g_clear_object(&self->down_button);
+	g_clear_object(&self->about_button);
 	g_clear_object(&self->prefs_stack);
 	G_OBJECT_CLASS(vala_panel_toplevel_config_parent_class)->finalize(obj);
 }
@@ -240,6 +272,7 @@ static GObject *vala_panel_configure_dialog_constructor(GType type, guint n_cons
 	                       G_BINDING_SYNC_CREATE);
 	/* plugin list */
 	init_plugin_list(self);
+	on_add_plugin_stack_box_generate(self);
 	gtk_widget_insert_action_group(self, "conf", conf);
 	gtk_widget_insert_action_group(self, "win", self->_toplevel);
 	gtk_widget_insert_action_group(self, "app", gtk_window_get_application(self->_toplevel));
@@ -276,365 +309,382 @@ static void vala_panel_toplevel_config_set_property(GObject *object, guint prope
 	}
 }
 
-static void on_sel_plugin_changed(GtkTreeSelection *tree_sel, void *data)
+static ValaPanelAppletInfo *get_info_from_applet(ValaPanelApplet *pl)
 {
-	ValaPanelToplevelConfig *self = VALA_PANEL_TOPLEVEL_CONFIG(data);
-	GtkTreeIter it;
-	GtkTreeModel *model;
-	ValaPanelApplet *pl;
-	if (gtk_tree_selection_get_selected(tree_sel, &model, &it))
-	{
-		gtk_tree_model_get(model, &it, COLUMN_DATA, &pl, -1);
-		ValaPanelAppletInfo *apl = vala_panel_applet_manager_get_applet_info(
-		    vala_panel_layout_get_manager(), pl, vala_panel_toplevel_get_core_settings());
-		const char *desc = vala_panel_applet_info_get_description(apl);
-		gtk_label_set_text(self->plugin_desc, desc);
-		gtk_widget_set_sensitive(self->configure_button,
-		                         vala_panel_applet_is_configurable(pl));
-	}
+	return vp_applet_manager_get_applet_info(vala_panel_layout_get_manager(),
+	                                         pl,
+	                                         vala_panel_toplevel_get_core_settings());
 }
-static void on_plugin_expand_toggled(GtkCellRendererToggle *render, const char *path, void *data)
+
+static GtkListBoxRow *create_info_representation(ValaPanelAppletInfo *pl_info)
 {
-	ValaPanelToplevelConfig *self = VALA_PANEL_TOPLEVEL_CONFIG(data);
-	GtkTreeIter it;
-	g_autoptr(GtkTreePath) tp = gtk_tree_path_new_from_string(path);
-	GtkTreeModel *model       = gtk_tree_view_get_model(self->plugin_list);
-	if (gtk_tree_model_get_iter(model, &it, tp))
-	{
-		ValaPanelApplet *pl;
-		bool expand;
-		gtk_tree_model_get(model, &it, COLUMN_EXPAND, &expand, COLUMN_DATA, &pl, -1);
-		ValaPanelAppletInfo *pl_info = vala_panel_applet_manager_get_applet_info(
-		    vala_panel_layout_get_manager(), pl, vala_panel_toplevel_get_core_settings());
-		if (vala_panel_applet_info_is_expandable(pl_info))
-		{
-			expand = !expand;
-			gtk_list_store_set(model, &it, COLUMN_EXPAND, expand, -1);
-			ValaPanelUnitSettings *s = vala_panel_layout_get_applet_settings(pl);
-			g_settings_set_boolean(s->common, VP_KEY_EXPAND, expand);
-		}
-	}
+	GtkListBoxRow *row = gtk_list_box_row_new();
+	GtkBox *box        = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 2);
+	gtk_widget_show(box);
+	gtk_container_add(row, box);
+
+	gtk_widget_set_margin_top(box, 4);
+	gtk_widget_set_margin_bottom(box, 4);
+
+	GtkImage *image =
+	    GTK_IMAGE(gtk_image_new_from_icon_name(vala_panel_applet_info_get_icon_name(pl_info),
+	                                           GTK_ICON_SIZE_BUTTON));
+	gtk_widget_set_margin_start(image, 12);
+	gtk_widget_set_margin_end(image, 14);
+	gtk_widget_show(image);
+	gtk_box_pack_start(box, image, false, false, 0);
+
+	GtkLabel *label = gtk_label_new(vala_panel_applet_info_get_name(pl_info));
+	gtk_widget_set_margin_end(label, 18);
+	gtk_widget_set_halign(label, GTK_ALIGN_START);
+	gtk_widget_show(label);
+	gtk_box_pack_start(box, label, false, false, 0);
+	gtk_widget_show(box);
+	gtk_widget_show(row);
+	return row;
 }
-static void on_stretch_render(GtkCellLayout *layout, GtkCellRenderer *renderer, GtkTreeModel *model,
-                              GtkTreeIter *iter, void *data)
+
+static GtkListBoxRow *create_applet_representation(ValaPanelToplevelConfig *self,
+                                                   ValaPanelApplet *pl)
 {
-	/* Set the control visible depending on whether stretch is available for the plugin.
-	 * The g_object_set method is touchy about its parameter, so we can't pass the boolean
-	 * directly. */
-	ValaPanelToplevelConfig *self = VALA_PANEL_TOPLEVEL_CONFIG(data);
-	ValaPanelApplet *pl;
-	gtk_tree_model_get(model, iter, COLUMN_DATA, &pl, -1);
 	ValaPanelAppletInfo *pl_info =
-	    vala_panel_applet_manager_get_applet_info(vala_panel_layout_get_manager(),
-	                                              pl,
-	                                              vala_panel_toplevel_get_core_settings());
-	gtk_cell_renderer_set_visible(renderer, vala_panel_applet_info_is_expandable(pl_info));
+	    vp_applet_manager_get_applet_info(vala_panel_layout_get_manager(),
+	                                      pl,
+	                                      vala_panel_toplevel_get_core_settings());
+	GtkListBoxRow *row = create_info_representation(pl_info);
+	GtkBox *box        = GTK_BOX(gtk_bin_get_child(row));
+	GtkButton *button  = GTK_BUTTON(gtk_button_new());
+	GtkImage *image = gtk_image_new_from_icon_name("list-remove-symbolic", GTK_ICON_SIZE_MENU);
+	gtk_widget_show(image);
+	gtk_container_add(button, image);
+	gtk_widget_show(button);
+	GtkStyleContext *context = gtk_widget_get_style_context(GTK_WIDGET(button));
+	gtk_style_context_add_class(context, "circular");
+	gtk_style_context_add_class(context, "destructive-action");
+	gtk_button_set_relief(button, GTK_RELIEF_NONE);
+	gtk_box_pack_end(box, button, false, false, 0);
+	g_signal_connect(button, "clicked", on_remove_plugin, self);
+	config_row_set_applet(row, pl);
+	return row;
 }
 
-static void update_plugin_list_model(ValaPanelToplevelConfig *self)
+/**
+ * Sort the list in accordance with pack and actual position
+ */
+static int plugin_list_sort(GtkListBoxRow *before, GtkListBoxRow *after, void *user_data)
 {
-	GtkTreeIter it;
-	GtkListStore *list = gtk_list_store_new(4,
-	                                        G_TYPE_STRING,
-	                                        G_TYPE_STRING,
-	                                        G_TYPE_BOOLEAN,
-	                                        VALA_PANEL_TYPE_APPLET);
-	GList *plugins =
-	    vala_panel_layout_get_applets_list(vala_panel_toplevel_get_layout(self->_toplevel));
-	for (GList *l = plugins; l != NULL; l = g_list_next(l))
+	ValaPanelLayout *layout      = VALA_PANEL_LAYOUT(user_data);
+	ValaPanelApplet *before_info = before ? config_row_get_applet(before) : NULL;
+	ValaPanelApplet *after_info  = after ? config_row_get_applet(after) : NULL;
+	uint bpos                    = vala_panel_layout_get_applet_position(layout, before_info);
+	uint apos                    = vala_panel_layout_get_applet_position(layout, after_info);
+	int bi                       = vala_panel_layout_get_applet_pack_type(before_info);
+
+	if (before_info != NULL && after_info != NULL &&
+	    vala_panel_layout_get_applet_pack_type(before_info) !=
+	        vala_panel_layout_get_applet_pack_type(after_info))
 	{
-		ValaPanelApplet *w = VALA_PANEL_APPLET(l->data);
-		bool expand        = gtk_widget_get_hexpand(w) && gtk_widget_get_vexpand(w);
-		gtk_list_store_append(list, &it);
-		ValaPanelAppletInfo *pl_info = vala_panel_applet_manager_get_applet_info(
-		    vala_panel_layout_get_manager(), w, vala_panel_toplevel_get_core_settings());
-		const char *name = vala_panel_applet_info_get_name(pl_info);
-		const char *icon = vala_panel_applet_info_get_icon_name(pl_info);
-		gtk_list_store_set(list,
-		                   &it,
-		                   COLUMN_ICON,
-		                   icon,
-		                   COLUMN_NAME,
-		                   _(name),
-		                   COLUMN_EXPAND,
-		                   expand,
-		                   COLUMN_DATA,
-		                   w,
-		                   -1);
+		int ai = vala_panel_layout_get_applet_pack_type(after_info);
+		if (ai != bi)
+		{
+			if (bi == PACK_START || ai == PACK_END)
+				return -1;
+			else if (ai == PACK_START || bi == PACK_END)
+				return 1;
+		}
+		return 0;
 	}
-	gtk_tree_view_set_model(self->plugin_list, list);
+
+	if (after_info == NULL)
+		return 0;
+
+	if (bpos < apos)
+		return -1;
+	else if (bpos > apos)
+		return 1;
+
+	return 0;
 }
 
-static void on_plugin_list_row_activated(GtkTreeView *tree_view, GtkTreePath *path,
-                                         GtkTreeViewColumn *column, gpointer user_data)
+/**
+ * Provide headers in the list to separate the visual positions
+ */
+static void plugin_list_headers(GtkListBoxRow *before, GtkListBoxRow *after, void *user_data)
+{
+	ValaPanelApplet *before_info = before ? config_row_get_applet(before) : NULL;
+	ValaPanelApplet *after_info  = after ? config_row_get_applet(after) : NULL;
+	int prev        = before_info ? vala_panel_layout_get_applet_pack_type(before_info) : -1;
+	int next        = after_info ? vala_panel_layout_get_applet_pack_type(after_info) : -1;
+	GtkLabel *label = GTK_LABEL(gtk_list_box_row_get_header(before));
+
+	if (after == NULL || prev != next)
+	{
+		if (!label)
+			label = GTK_LABEL(gtk_label_new(""));
+		switch (prev)
+		{
+		case PACK_START:
+			gtk_label_set_text(label, _("Start"));
+			break;
+		case PACK_CENTER:
+			gtk_label_set_text(label, _("Center"));
+			break;
+		default:
+			gtk_label_set_text(label, _("End"));
+			break;
+		}
+		GtkStyleContext *context = gtk_widget_get_style_context(GTK_WIDGET(label));
+		gtk_style_context_add_class(context, "dim-label");
+		gtk_style_context_add_class(context, "applet-row-header");
+		gtk_widget_set_halign(GTK_WIDGET(label), GTK_ALIGN_START);
+		gtk_widget_set_valign(GTK_WIDGET(label), GTK_ALIGN_CENTER);
+		gtk_widget_set_margin_start(GTK_WIDGET(label), 4);
+		gtk_widget_set_margin_top(GTK_WIDGET(label), 2);
+		gtk_widget_set_margin_bottom(GTK_WIDGET(label), 2);
+		gtk_label_set_use_markup(label, true);
+		gtk_list_box_row_set_header(before, GTK_WIDGET(label));
+	}
+	else
+		gtk_list_box_row_set_header(before, NULL);
+}
+
+static void on_plugin_list_row_selected(GtkListBox *box, GtkListBoxRow *row, gpointer data)
+{
+	ValaPanelToplevelConfig *self = VALA_PANEL_TOPLEVEL_CONFIG(data);
+	if (!row)
+	{
+		gtk_widget_set_sensitive(self->about_button, false);
+		gtk_widget_set_sensitive(self->up_button, false);
+		gtk_widget_set_sensitive(self->down_button, false);
+		gtk_stack_set_visible_child_name(self->applet_info_stack, NO_SETTINGS);
+	}
+	else
+	{
+		ValaPanelApplet *pl     = config_row_get_applet(row);
+		ValaPanelLayout *layout = vala_panel_toplevel_get_layout(self->_toplevel);
+		gtk_widget_set_sensitive(self->about_button, true);
+		GtkListBoxRow *down_row =
+		    gtk_list_box_get_row_at_index(self->plugin_list,
+		                                  gtk_list_box_row_get_index(row) + 1);
+		ValaPanelApplet *down = down_row ? config_row_get_applet(down_row) : NULL;
+		GtkListBoxRow *up_row =
+		    gtk_list_box_get_row_at_index(self->plugin_list,
+		                                  gtk_list_box_row_get_index(row) - 1);
+		ValaPanelApplet *up = up_row ? config_row_get_applet(up_row) : NULL;
+		gtk_widget_set_sensitive(self->up_button,
+		                         vala_panel_layout_can_move_to_direction(layout,
+		                                                                 pl,
+		                                                                 up,
+		                                                                 GTK_PACK_START));
+		gtk_widget_set_sensitive(self->down_button,
+		                         vala_panel_layout_can_move_to_direction(layout,
+		                                                                 pl,
+		                                                                 down,
+		                                                                 GTK_PACK_END));
+		if (vala_panel_applet_is_configurable(pl))
+			gtk_stack_set_visible_child_name(self->applet_info_stack,
+			                                 vala_panel_applet_get_uuid(pl));
+		else
+			gtk_stack_set_visible_child_name(self->applet_info_stack, NO_SETTINGS);
+	}
+}
+static void plugin_list_generate_applet_settings(ValaPanelApplet *pl, ValaPanelToplevelConfig *self)
 {
-	ValaPanelToplevelConfig *self = VALA_PANEL_TOPLEVEL_CONFIG(user_data);
-	GtkTreeSelection *tree_sel    = gtk_tree_view_get_selection(tree_view);
-	GtkTreeModel *model;
-	GtkTreeIter iter;
-	ValaPanelApplet *pl;
-	if (!gtk_tree_selection_get_selected(tree_sel, &model, &iter))
-		return;
-	gtk_tree_model_get(model, &iter, COLUMN_DATA, &pl, -1);
 	if (vala_panel_applet_is_configurable(pl))
-		vala_panel_applet_show_config_dialog(pl);
+		gtk_stack_add_named(self->applet_info_stack,
+		                    vala_panel_applet_get_settings_ui(pl),
+		                    vala_panel_applet_get_uuid(pl));
+}
+
+static void plugin_list_add_applet(const char *type, ValaPanelToplevelConfig *self,
+                                   GtkListBoxRow *prev)
+{
+	ValaPanelLayout *layout  = vala_panel_toplevel_get_layout(self->_toplevel);
+	ValaPanelApplet *prev_pl = prev ? config_row_get_applet(prev) : NULL;
+	ValaPanelAppletPackType prev_pack =
+	    prev ? vala_panel_layout_get_applet_pack_type(prev_pl) : PACK_START;
+	uint prev_pos       = prev ? vala_panel_layout_get_applet_position(layout, prev_pl) : 0;
+	ValaPanelApplet *pl = vala_panel_layout_insert_applet(layout, type, prev_pack, prev_pos);
+	GtkListBoxRow *row  = create_applet_representation(self, pl);
+	gtk_container_add(self->plugin_list, row);
+	plugin_list_generate_applet_settings(pl, self);
+	gtk_list_box_invalidate_sort(self->plugin_list);
 }
 
 static void init_plugin_list(ValaPanelToplevelConfig *self)
 {
-	GtkTreeIter it;
-	GtkCellRenderer *textrender = gtk_cell_renderer_pixbuf_new();
-	GtkTreeViewColumn *col      = gtk_tree_view_column_new_with_attributes(_("Icon"),
-                                                                          textrender,
-                                                                          "icon-name",
-                                                                          COLUMN_ICON,
-                                                                          NULL);
-	gtk_tree_view_column_set_expand(col, true);
-	gtk_tree_view_append_column(self->plugin_list, col);
-	textrender = gtk_cell_renderer_text_new();
-	col        = gtk_tree_view_column_new_with_attributes(_("Currently loaded plugins"),
-                                                       textrender,
-                                                       "text",
-                                                       COLUMN_NAME,
-                                                       NULL);
-	gtk_tree_view_column_set_expand(col, true);
-	gtk_tree_view_append_column(self->plugin_list, col);
-	GtkCellRendererToggle *render = gtk_cell_renderer_toggle_new();
-	gtk_cell_renderer_toggle_set_activatable(render, true);
-	g_signal_connect(render, "toggled", G_CALLBACK(on_plugin_expand_toggled), self);
-	col = gtk_tree_view_column_new_with_attributes(_("Stretch"),
-	                                               render,
-	                                               "active",
-	                                               COLUMN_EXPAND,
-	                                               NULL);
-	gtk_tree_view_column_set_expand(col, true);
-	gtk_tree_view_column_set_cell_data_func(col, render, on_stretch_render, self, NULL);
-	gtk_tree_view_append_column(self->plugin_list, col);
-	update_plugin_list_model(self);
-	g_signal_connect(self->plugin_list,
-	                 "row-activated",
-	                 G_CALLBACK(on_plugin_list_row_activated),
-	                 self);
-	GtkTreeModel *list         = gtk_tree_view_get_model(self->plugin_list);
-	GtkTreeSelection *tree_sel = gtk_tree_view_get_selection(self->plugin_list);
-	gtk_tree_selection_set_mode(tree_sel, GTK_SELECTION_BROWSE);
-	g_signal_connect(tree_sel, "changed", G_CALLBACK(on_sel_plugin_changed), self);
-	if (gtk_tree_model_get_iter_first(list, &it))
-		gtk_tree_selection_select_iter(tree_sel, &it);
+	GList *plugins =
+	    vala_panel_layout_get_applets_list(vala_panel_toplevel_get_layout(self->_toplevel));
+	GtkListBoxRow *prev;
+	for (GList *l = plugins; l != NULL; l = g_list_next(l))
+	{
+		ValaPanelApplet *w = VALA_PANEL_APPLET(l->data);
+		GtkListBoxRow *r   = create_applet_representation(self, w);
+		gtk_container_add(GTK_CONTAINER(self->plugin_list), GTK_WIDGET(r));
+		plugin_list_generate_applet_settings(w, self);
+	}
+	gtk_list_box_set_sort_func(self->plugin_list,
+	                           plugin_list_sort,
+	                           vala_panel_toplevel_get_layout(self->_toplevel),
+	                           NULL);
+	gtk_list_box_set_header_func(self->plugin_list, plugin_list_headers, self, NULL);
+	on_plugin_list_row_selected(self->plugin_list, NULL, self);
 }
 
 static void on_about_plugin(GtkButton *btn, void *data)
 {
 	ValaPanelToplevelConfig *self = VALA_PANEL_TOPLEVEL_CONFIG(data);
-	GtkTreeSelection *tree_sel    = gtk_tree_view_get_selection(self->plugin_list);
-	GtkTreeModel *model;
-	GtkTreeIter iter;
-	ValaPanelApplet *pl;
-	if (!gtk_tree_selection_get_selected(tree_sel, &model, &iter))
+	GtkListBoxRow *row            = gtk_list_box_get_selected_row(self->plugin_list);
+	if (!row)
 		return;
-	gtk_tree_model_get(model, &iter, COLUMN_DATA, &pl, -1);
-	ValaPanelAppletInfo *info =
-	    vala_panel_applet_manager_get_applet_info(vala_panel_layout_get_manager(),
-	                                              pl,
-	                                              vala_panel_toplevel_get_core_settings());
+	ValaPanelApplet *pl       = config_row_get_applet(row);
+	ValaPanelAppletInfo *info = get_info_from_applet(pl);
 	vala_panel_applet_info_show_about_dialog(info);
 }
 
-static void on_configure_plugin(GtkButton *btn, void *data)
+static int listbox_new_name_sort(GtkListBoxRow *before, GtkListBoxRow *after, void *user_data)
 {
-	ValaPanelToplevelConfig *self = VALA_PANEL_TOPLEVEL_CONFIG(data);
-	GtkTreeSelection *tree_sel    = gtk_tree_view_get_selection(self->plugin_list);
-	GtkTreeModel *model;
-	GtkTreeIter iter;
-	ValaPanelApplet *pl;
-	if (!gtk_tree_selection_get_selected(tree_sel, &model, &iter))
-		return;
-	gtk_tree_model_get(model, &iter, COLUMN_DATA, &pl, -1);
-	if (vala_panel_applet_is_configurable(pl))
-		vala_panel_applet_show_config_dialog(pl);
+	AppletInfoData *before_info = before ? config_row_get_info_data(before) : NULL;
+	AppletInfoData *after_info  = after ? config_row_get_info_data(after) : NULL;
+	if (before_info && after_info)
+		return g_strcmp0(vala_panel_applet_info_get_name(before_info->info),
+		                 vala_panel_applet_info_get_name(after_info->info));
+	else if (before_info)
+		return -1;
+	else
+		return 1;
 }
-static int sort_by_name(GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, void *unused)
+
+static void listbox_new_applet_row_activated(GtkListBox *box, GtkListBoxRow *row,
+                                             gpointer user_data)
 {
-	char *str_a, *str_b;
-	gtk_tree_model_get(model, a, 0, &str_a, -1);
-	gtk_tree_model_get(model, b, 0, &str_b, -1);
-	int ret = g_utf8_collate(str_a, str_b);
-	g_free(str_a);
-	g_free(str_b);
+	ValaPanelToplevelConfig *self = VALA_PANEL_TOPLEVEL_CONFIG(user_data);
+	const char *type =
+	    vala_panel_applet_info_get_module_name(config_row_get_info_data(row)->info);
+	GtkListBoxRow *prev = gtk_list_box_get_selected_row(self->plugin_list);
+	plugin_list_add_applet(type, self, prev);
+	gtk_list_box_invalidate_filter(self->listbox_new_applet);
+}
+
+static int listbox_new_filter(GtkListBoxRow *row, void *data)
+{
+	AppletInfoData *d = config_row_get_info_data(row);
+	bool ret          = false;
+	if ((d->count < 1) || !vala_panel_applet_info_is_exclusive(d->info))
+		ret = true;
 	return ret;
 }
 
-static void update_widget_position_keys(ValaPanelToplevelConfig *self)
+static void destroy(GtkWidget *applet, G_GNUC_UNUSED void *data)
 {
-	ValaPanelLayout *layout       = vala_panel_toplevel_get_layout(self->_toplevel);
-	g_autoptr(GList) applets_list = vala_panel_layout_get_applets_list(layout);
-	for (GList *l = applets_list; l != NULL; l = g_list_next(l))
-	{
-		ValaPanelApplet *applet  = VALA_PANEL_APPLET(l->data);
-		uint idx                 = vala_panel_layout_get_applet_position(layout, applet);
-		ValaPanelUnitSettings *s = vala_panel_layout_get_applet_settings(applet);
-		g_settings_set_uint(s->common, VP_KEY_POSITION, idx);
-	}
+	g_clear_pointer(&applet, gtk_widget_destroy);
 }
 
-static void on_add_plugin_row_activated(GtkTreeView *tree_view, GtkTreePath *path,
-                                        GtkTreeViewColumn *column, gpointer user_data)
+static void available_plugins_reload(ValaPanelToplevelConfig *self)
 {
-	ValaPanelToplevelConfig *self = VALA_PANEL_TOPLEVEL_CONFIG(user_data);
-	GtkTreeIter it;
-	GtkTreeModel *model;
-	GtkTreeSelection *sel = gtk_tree_view_get_selection(tree_view);
-	if (gtk_tree_selection_get_selected(sel, &model, &it))
+	/* Populate list of available plugins.
+	 * Omit plugins that can only exist once per system if it is already configured. */
+	vp_applet_manager_reload_applets(vala_panel_layout_get_manager());
+	gtk_container_foreach(GTK_CONTAINER(self->listbox_new_applet), (GtkCallback)destroy, NULL);
+	GList *all_types = vp_applet_manager_get_all_types(vala_panel_layout_get_manager());
+	for (GList *l = all_types; l != NULL; l = g_list_next(l))
 	{
-		const char *type;
-		gtk_tree_model_get(model, &it, 2, &type, -1);
-		ValaPanelLayout *layout = vala_panel_toplevel_get_layout(self->_toplevel);
-		vala_panel_layout_add_applet(layout, type);
-		vala_panel_layout_update_applet_positions(layout);
-		update_plugin_list_model(self);
+		AppletInfoData *d  = (AppletInfoData *)l->data;
+		GtkListBoxRow *row = create_info_representation(d->info);
+		config_row_set_info(row, d);
+		gtk_container_add(self->listbox_new_applet, row);
 	}
-	update_widget_position_keys(self);
+	g_list_free(all_types);
+}
+
+static void on_add_plugin_stack_box_generate(ValaPanelToplevelConfig *self)
+{
+	available_plugins_reload(self);
+	gtk_list_box_set_sort_func(self->listbox_new_applet, listbox_new_name_sort, self, NULL);
+	gtk_list_box_set_filter_func(self->listbox_new_applet,
+	                             (GtkListBoxFilterFunc)listbox_new_filter,
+	                             self,
+	                             NULL);
 }
 
 static void on_add_plugin(GtkButton *btn, ValaPanelToplevelConfig *self)
 {
-	GtkPopover *dlg           = gtk_popover_new(self->adding_button);
-	GtkScrolledWindow *scroll = gtk_scrolled_window_new(NULL, NULL);
-	gtk_scrolled_window_set_shadow_type(scroll, GTK_SHADOW_IN);
-	gtk_scrolled_window_set_policy(scroll, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_container_add(dlg, scroll);
-	gtk_widget_show(scroll);
-	GtkTreeView *view = gtk_tree_view_new();
-	gtk_container_add(scroll, view);
-	gtk_widget_show(view);
-	GtkTreeSelection *tree_sel = gtk_tree_view_get_selection(view);
-	gtk_tree_selection_set_mode(tree_sel, GTK_SELECTION_BROWSE);
-
-	GtkCellRenderer *render = gtk_cell_renderer_pixbuf_new();
-	GtkTreeViewColumn *col  = gtk_tree_view_column_new_with_attributes(_("Icon"),
-                                                                          render,
-                                                                          "icon-name",
-                                                                          COLUMN_ICON,
-                                                                          NULL);
-
-	gtk_tree_view_append_column(view, col);
-	render = gtk_cell_renderer_text_new();
-	col    = gtk_tree_view_column_new_with_attributes(_("Available plugins"),
-                                                       render,
-                                                       "text",
-                                                       1,
-                                                       NULL);
-	gtk_tree_view_append_column(view, col);
-
-	GtkListStore *list = gtk_list_store_new(3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
-
-	/* Populate list of available plugins.
-	 * Omit plugins that can only exist once per system if it is already configured. */
-	vala_panel_applet_manager_reload_applets(vala_panel_layout_get_manager());
-	GList *all_types = vala_panel_applet_manager_get_all_types(vala_panel_layout_get_manager());
-	for (GList *l = all_types; l != NULL; l = g_list_next(l))
+	const char *name = gtk_stack_get_visible_child_name(self->applet_info_stack);
+	if (!g_strcmp0(name, NEW_APPLET_NAME))
 	{
-		AppletInfoData *d = (AppletInfoData *)l->data;
-		if (!vala_panel_applet_info_is_exclusive(d->info) || (d->count < 1))
-		{
-			GtkTreeIter it;
-			gtk_list_store_append(list, &it);
-			/* it is safe to put classes data here - they will be valid until restart */
-			const char *icon_name   = vala_panel_applet_info_get_icon_name(d->info);
-			const char *name        = _(vala_panel_applet_info_get_name(d->info));
-			const char *module_name = vala_panel_applet_info_get_module_name(d->info);
-
-			gtk_list_store_set(list, &it, 0, icon_name, 1, name, 2, module_name, -1);
-		}
+		GtkListBoxRow *row = gtk_list_box_get_selected_row(self->plugin_list);
+		on_plugin_list_row_selected(self->plugin_list, row, self);
+		return;
 	}
-	g_list_free(all_types);
-	gtk_tree_sortable_set_default_sort_func(list, sort_by_name, self, NULL);
-	gtk_tree_sortable_set_sort_column_id(list,
-	                                     GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID,
-	                                     GTK_SORT_ASCENDING);
-	gtk_tree_view_set_activate_on_single_click(view, true);
-	gtk_tree_view_set_model(view, list);
-	g_signal_connect(view, "row-activated", G_CALLBACK(on_add_plugin_row_activated), self);
-	gtk_scrolled_window_set_min_content_width(scroll, 320);
-	gtk_scrolled_window_set_min_content_height(scroll, 200);
-	gtk_widget_show(dlg);
+	available_plugins_reload(self);
+	gtk_widget_set_sensitive(self->about_button, false);
+	gtk_widget_set_sensitive(self->up_button, false);
+	gtk_widget_set_sensitive(self->down_button, false);
+	gtk_stack_set_visible_child_name(self->applet_info_stack, NEW_APPLET_NAME);
+}
+
+static inline GtkListBoxRow *get_row_from_btn(GtkButton *self)
+{
+	return GTK_LIST_BOX_ROW(gtk_widget_get_ancestor(GTK_WIDGET(self), GTK_TYPE_LIST_BOX_ROW));
 }
 
 static void on_remove_plugin(GtkButton *btn, void *user_data)
 {
 	ValaPanelToplevelConfig *self = VALA_PANEL_TOPLEVEL_CONFIG(user_data);
-	GtkTreeIter it;
-	GtkTreeModel *model;
-	GtkTreeSelection *tree_sel = gtk_tree_view_get_selection(self->plugin_list);
-	ValaPanelApplet *pl;
-	if (gtk_tree_selection_get_selected(tree_sel, &model, &it))
-	{
-		GtkTreePath *tree_path = gtk_tree_model_get_path(model, &it);
-		gtk_tree_model_get(model, &it, COLUMN_DATA, &pl, -1);
-		if (gtk_tree_path_get_indices(tree_path)[0] >=
-		    gtk_tree_model_iter_n_children(model, NULL))
-			gtk_tree_path_prev(tree_path);
-		gtk_list_store_remove(GTK_LIST_STORE(model), &it);
-		gtk_tree_selection_select_path(tree_sel, tree_path);
-		ValaPanelLayout *layout = vala_panel_toplevel_get_layout(self->_toplevel);
-		vala_panel_layout_remove_applet(layout, pl);
-	}
+	GtkListBoxRow *row            = get_row_from_btn(btn);
+	if (!row)
+		return;
+	ValaPanelApplet *pl = config_row_get_applet(row);
+	GtkWidget *w =
+	    gtk_stack_get_child_by_name(self->applet_info_stack, vala_panel_applet_get_uuid(pl));
+	int index              = gtk_list_box_row_get_index(row);
+	GtkListBoxRow *sel_row = gtk_list_box_get_selected_row(self->plugin_list);
+	int sel_index          = sel_row ? gtk_list_box_row_get_index(sel_row) : -1;
+	gtk_widget_destroy0(row);
+	gtk_widget_destroy0(w);
+	if (index == sel_index)
+		gtk_list_box_select_row(self->plugin_list,
+		                        gtk_list_box_get_row_at_index(self->plugin_list, index));
+	ValaPanelLayout *layout      = vala_panel_toplevel_get_layout(self->_toplevel);
+	ValaPanelAppletPackType pack = vala_panel_layout_get_applet_pack_type(pl);
+	uint pos                     = vala_panel_layout_get_applet_position(layout, pl);
+	vala_panel_layout_remove_applet(layout, pl);
+	gtk_list_box_invalidate_sort(self->plugin_list);
+	gtk_list_box_invalidate_filter(self->listbox_new_applet);
 }
 
 static void on_moveup_plugin(GtkButton *btn, void *user_data)
 {
 	ValaPanelToplevelConfig *self = VALA_PANEL_TOPLEVEL_CONFIG(user_data);
-	GtkTreeIter it, prev;
-	GtkTreeModel *model        = gtk_tree_view_get_model(self->plugin_list);
-	GtkTreeSelection *tree_sel = gtk_tree_view_get_selection(self->plugin_list);
-	if (!gtk_tree_model_get_iter_first(model, &it))
+	ValaPanelLayout *layout       = vala_panel_toplevel_get_layout(self->_toplevel);
+	GtkListBoxRow *row            = gtk_list_box_get_selected_row(self->plugin_list);
+	if (!row)
 		return;
-	if (gtk_tree_selection_iter_is_selected(tree_sel, &it))
-		return;
-	do
-	{
-		if (gtk_tree_selection_iter_is_selected(tree_sel, &it))
-		{
-			ValaPanelApplet *pl;
-			ValaPanelLayout *layout = vala_panel_toplevel_get_layout(self->_toplevel);
-			gtk_tree_model_get(model, &it, COLUMN_DATA, &pl, -1);
-			gtk_list_store_move_before(GTK_LIST_STORE(model), &it, &prev);
-
-			uint i = vala_panel_layout_get_applet_position(layout, pl);
-			/* reorder in config, 0 is Global */
-			i = i > 0 ? i : 0;
-
-			/* reorder in panel */
-			vala_panel_layout_set_applet_position(layout, pl, (int)i - 1);
-			update_widget_position_keys(self);
-			return;
-		}
-		prev = it;
-	} while (gtk_tree_model_iter_next(model, &it));
+	ValaPanelApplet *pl = config_row_get_applet(row);
+	int index           = gtk_list_box_row_get_index(row);
+	index--;
+	GtkListBoxRow *nxt      = gtk_list_box_get_row_at_index(self->plugin_list, index);
+	ValaPanelApplet *nxt_pl = nxt ? config_row_get_applet(nxt) : NULL;
+	vala_panel_layout_move_applet_one_step(layout, pl, nxt_pl, GTK_PACK_START);
+	gtk_list_box_invalidate_headers(self->plugin_list);
+	gtk_list_box_invalidate_sort(self->plugin_list);
 }
 
 static void on_movedown_plugin(GtkButton *btn, void *user_data)
 {
 	ValaPanelToplevelConfig *self = VALA_PANEL_TOPLEVEL_CONFIG(user_data);
-	GtkTreeIter it, next;
-	GtkTreeModel *model;
-	GtkTreeSelection *tree_sel = gtk_tree_view_get_selection(self->plugin_list);
-	if (!gtk_tree_selection_get_selected(tree_sel, &model, &it))
+	ValaPanelLayout *layout       = vala_panel_toplevel_get_layout(self->_toplevel);
+	GtkListBoxRow *row            = gtk_list_box_get_selected_row(self->plugin_list);
+	if (!row)
 		return;
-	next = it;
-	if (!gtk_tree_model_iter_next(model, &next))
-		return;
-
-	ValaPanelApplet *pl;
-	ValaPanelLayout *layout = vala_panel_toplevel_get_layout(self->_toplevel);
-	gtk_tree_model_get(model, &it, COLUMN_DATA, &pl, -1);
-	gtk_list_store_move_after(GTK_LIST_STORE(model), &it, &next);
-
-	uint i = vala_panel_layout_get_applet_position(layout, pl);
-	/* reorder in panel */
-	vala_panel_layout_set_applet_position(layout, pl, (int)i + 1);
-	update_widget_position_keys(self);
-	return;
+	ValaPanelApplet *pl = config_row_get_applet(row);
+	int index           = gtk_list_box_row_get_index(row);
+	index++;
+	GtkListBoxRow *nxt      = gtk_list_box_get_row_at_index(self->plugin_list, index);
+	ValaPanelApplet *nxt_pl = nxt ? config_row_get_applet(nxt) : NULL;
+	vala_panel_layout_move_applet_one_step(layout, pl, nxt_pl, GTK_PACK_END);
+	gtk_list_box_invalidate_headers(self->plugin_list);
+	gtk_list_box_invalidate_sort(self->plugin_list);
 }
 
 static void vala_panel_toplevel_config_class_init(ValaPanelToplevelConfigClass *klass)
@@ -712,20 +762,35 @@ static void vala_panel_toplevel_config_class_init(ValaPanelToplevelConfigClass *
 	                                          G_STRUCT_OFFSET(ValaPanelToplevelConfig,
 	                                                          plugin_list));
 	gtk_widget_class_bind_template_child_full(GTK_WIDGET_CLASS(klass),
-	                                          "plugin-desc",
+	                                          "applet-info-stack",
 	                                          FALSE,
 	                                          G_STRUCT_OFFSET(ValaPanelToplevelConfig,
-	                                                          plugin_desc));
+	                                                          applet_info_stack));
+	gtk_widget_class_bind_template_child_full(GTK_WIDGET_CLASS(klass),
+	                                          "listbox-new-applet",
+	                                          FALSE,
+	                                          G_STRUCT_OFFSET(ValaPanelToplevelConfig,
+	                                                          listbox_new_applet));
 	gtk_widget_class_bind_template_child_full(GTK_WIDGET_CLASS(klass),
 	                                          "add-button",
 	                                          FALSE,
 	                                          G_STRUCT_OFFSET(ValaPanelToplevelConfig,
 	                                                          adding_button));
 	gtk_widget_class_bind_template_child_full(GTK_WIDGET_CLASS(klass),
-	                                          "configure-button",
+	                                          "up-button",
 	                                          FALSE,
 	                                          G_STRUCT_OFFSET(ValaPanelToplevelConfig,
-	                                                          configure_button));
+	                                                          up_button));
+	gtk_widget_class_bind_template_child_full(GTK_WIDGET_CLASS(klass),
+	                                          "down-button",
+	                                          FALSE,
+	                                          G_STRUCT_OFFSET(ValaPanelToplevelConfig,
+	                                                          down_button));
+	gtk_widget_class_bind_template_child_full(GTK_WIDGET_CLASS(klass),
+	                                          "about-button",
+	                                          FALSE,
+	                                          G_STRUCT_OFFSET(ValaPanelToplevelConfig,
+	                                                          about_button));
 	gtk_widget_class_bind_template_child_full(GTK_WIDGET_CLASS(klass),
 	                                          "prefs",
 	                                          FALSE,
@@ -739,22 +804,22 @@ static void vala_panel_toplevel_config_class_init(ValaPanelToplevelConfigClass *
 	gtk_widget_class_bind_template_callback_full(GTK_WIDGET_CLASS(klass),
 	                                             "on_monitor_changed",
 	                                             G_CALLBACK(on_monitors_changed));
-	gtk_widget_class_bind_template_callback_full(GTK_WIDGET_CLASS(klass),
-	                                             "on_configure_plugin",
-	                                             G_CALLBACK(on_configure_plugin));
 	gtk_widget_class_bind_template_callback_full(GTK_WIDGET_CLASS(klass),
 	                                             "on_about_plugin",
 	                                             G_CALLBACK(on_about_plugin));
 	gtk_widget_class_bind_template_callback_full(GTK_WIDGET_CLASS(klass),
 	                                             "on_add_plugin",
 	                                             G_CALLBACK(on_add_plugin));
-	gtk_widget_class_bind_template_callback_full(GTK_WIDGET_CLASS(klass),
-	                                             "on_remove_plugin",
-	                                             G_CALLBACK(on_remove_plugin));
 	gtk_widget_class_bind_template_callback_full(GTK_WIDGET_CLASS(klass),
 	                                             "on_moveup_plugin",
 	                                             G_CALLBACK(on_moveup_plugin));
 	gtk_widget_class_bind_template_callback_full(GTK_WIDGET_CLASS(klass),
 	                                             "on_movedown_plugin",
 	                                             G_CALLBACK(on_movedown_plugin));
+	gtk_widget_class_bind_template_callback_full(GTK_WIDGET_CLASS(klass),
+	                                             "plugin_list_row_selected",
+	                                             G_CALLBACK(on_plugin_list_row_selected));
+	gtk_widget_class_bind_template_callback_full(GTK_WIDGET_CLASS(klass),
+	                                             "listbox_new_applet_row_activated",
+	                                             G_CALLBACK(listbox_new_applet_row_activated));
 }
