@@ -28,11 +28,19 @@
 #include <stdbool.h>
 
 static const int PERIOD = 200;
+typedef enum
+{
+	AH_HIDDEN  = 0,
+	AH_WAITING = 1,
+	AH_GRAB    = 2,
+	AH_VISIBLE = 3,
+} PanelAutohideState;
 
 static ValaPanelPlatform *platform = NULL;
 
 static ValaPanelToplevel *vala_panel_toplevel_create(GtkApplication *app, const char *name, int mon,
                                                      PanelGravity e);
+static void vala_panel_toplevel_update_geometry(ValaPanelToplevel *self);
 static void activate_new_panel(GSimpleAction *act, GVariant *param, void *data);
 static void activate_remove_panel(GSimpleAction *act, GVariant *param, void *data);
 static void activate_panel_settings(GSimpleAction *act, GVariant *param, void *data);
@@ -121,12 +129,12 @@ inline bool vala_panel_toplevel_is_initialized(ValaPanelToplevel *self)
 	return self->initialized;
 }
 
-G_GNUC_INTERNAL inline ValaPanelPlatform *vala_panel_toplevel_get_current_platform()
+G_GNUC_INTERNAL ValaPanelPlatform *vala_panel_toplevel_get_current_platform()
 {
 	return platform;
 }
 
-void stop_ui(ValaPanelToplevel *self)
+static void stop_ui(ValaPanelToplevel *self)
 {
 	if (self->pref_dialog != NULL)
 		gtk_dialog_response(self->pref_dialog, GTK_RESPONSE_CLOSE);
@@ -159,7 +167,7 @@ static void vala_panel_toplevel_finalize(GObject *obj)
 	G_OBJECT_CLASS(vala_panel_toplevel_parent_class)->finalize(obj);
 }
 
-void start_ui(ValaPanelToplevel *self)
+static void start_ui(ValaPanelToplevel *self)
 {
 	css_apply_from_resource(GTK_WIDGET(self),
 	                        "/org/vala-panel/lib/style.css",
@@ -205,7 +213,7 @@ void start_ui(ValaPanelToplevel *self)
 	self->initialized = true;
 }
 
-static void setup(ValaPanelToplevel *self, bool use_internal_values)
+static void init_actions(ValaPanelToplevel *self, bool use_internal_values)
 {
 	if (self->settings == NULL)
 	{
@@ -304,7 +312,8 @@ static GtkMenu *vala_panel_toplevel_get_plugin_menu(ValaPanelToplevel *self, Val
 	return self->context_menu;
 }
 
-bool vala_panel_toplevel_release_event_helper(GtkWidget *_sender, GdkEventButton *e, gpointer obj)
+G_GNUC_INTERNAL bool vala_panel_toplevel_release_event_helper(GtkWidget *_sender, GdkEventButton *e,
+                                                              gpointer obj)
 {
 	ValaPanelToplevel *self = VALA_PANEL_TOPLEVEL(obj);
 	ValaPanelApplet *pl     = NULL;
@@ -349,6 +358,20 @@ void vala_panel_toplevel_configure(ValaPanelToplevel *self, const char *page)
 	                         self);
 }
 
+G_GNUC_INTERNAL void vala_panel_toplevel_configure_applet(ValaPanelToplevel *self, const char *uuid)
+{
+	if (self->pref_dialog == NULL)
+		self->pref_dialog =
+		    g_object_new(vala_panel_toplevel_config_get_type(), "toplevel", self, NULL);
+	gtk_stack_set_visible_child_name(self->pref_dialog->prefs_stack, "applets");
+	vala_panel_toplevel_config_select_applet(self->pref_dialog, uuid);
+	gtk_window_present(self->pref_dialog);
+	g_signal_connect_swapped(self->pref_dialog,
+	                         "hide",
+	                         G_CALLBACK(vala_panel_toplevel_destroy_pref_dialog),
+	                         self);
+}
+
 static void activate_new_panel(GSimpleAction *act, GVariant *param, void *data)
 {
 	ValaPanelToplevel *self  = VALA_PANEL_TOPLEVEL(data);
@@ -449,7 +472,7 @@ static void activate_panel_settings(GSimpleAction *act, GVariant *param, void *d
 /**************************************************************************
  * Appearance -------------------------------------------------------------
  **************************************************************************/
-G_GNUC_INTERNAL void update_appearance(ValaPanelToplevel *self)
+static void update_appearance(ValaPanelToplevel *self)
 {
 	if (self->provider)
 		gtk_style_context_remove_provider(gtk_widget_get_style_context(GTK_WIDGET(self)),
@@ -571,7 +594,7 @@ static ValaPanelToplevel *vala_panel_toplevel_new_from_position(GtkApplication *
 	ValaPanelToplevel *ret = vala_panel_toplevel_create_window(app, uid);
 	ret->mon               = mon;
 	ret->gravity           = edge;
-	setup(ret, true);
+	init_actions(ret, true);
 	return ret;
 }
 static ValaPanelToplevel *vala_panel_toplevel_create(GtkApplication *app, const char *name, int mon,
@@ -589,7 +612,7 @@ ValaPanelToplevel *vala_panel_toplevel_new(GtkApplication *app, ValaPanelPlatfor
 	ValaPanelToplevel *ret = vala_panel_toplevel_create_window(app, uid);
 	if (platform == NULL)
 		platform = plt;
-	setup(ret, false);
+	init_actions(ret, false);
 	return ret;
 }
 
@@ -677,11 +700,25 @@ static void vala_panel_toplevel_update_geometry_no_orient(ValaPanelToplevel *sel
 		gtk_main_iteration();
 }
 
-void vala_panel_toplevel_update_geometry(ValaPanelToplevel *self)
+static void vala_panel_toplevel_update_geometry(ValaPanelToplevel *self)
 {
 	vala_panel_toplevel_update_geometry_no_orient(self);
 	g_object_notify(G_OBJECT(self), "orientation");
 }
+
+void vala_panel_update_visibility(ValaPanelToplevel *panel, int mons)
+{
+	int monitor;
+	g_object_get(panel, VP_KEY_MONITOR, &monitor, NULL);
+	if (monitor < mons && !vala_panel_toplevel_is_initialized(panel) && mons > 0)
+		start_ui(panel);
+	else if ((monitor >= mons && vala_panel_toplevel_is_initialized(panel)) || mons == 0)
+		stop_ui(panel);
+	else
+	{
+		vala_panel_toplevel_update_geometry(panel);
+	}
+}
 /****************************************************
  *         autohide : new version                   *
  ****************************************************/
@@ -694,11 +731,13 @@ static int timeout_func(ValaPanelToplevel *self)
 		vala_panel_toplevel_update_geometry_no_orient(self);
 		self->ah_state = AH_HIDDEN;
 	}
-	return false;
+	return G_SOURCE_REMOVE;
 }
 
 static void ah_show(ValaPanelToplevel *self)
 {
+	if (self->ah_state >= AH_GRAB)
+		return;
 	css_toggle_class(GTK_WIDGET(self), "-panel-transparent", false);
 	gtk_revealer_set_reveal_child(self->ah_rev, true);
 	vala_panel_toplevel_update_geometry_no_orient(self);
@@ -707,8 +746,10 @@ static void ah_show(ValaPanelToplevel *self)
 
 static void ah_hide(ValaPanelToplevel *self)
 {
+	if (self->ah_state <= AH_GRAB)
+		return;
 	self->ah_state = AH_WAITING;
-	g_timeout_add(PERIOD, (GSourceFunc)timeout_func, self);
+	g_timeout_add_full(G_PRIORITY_HIGH, PERIOD, (GSourceFunc)timeout_func, self, NULL);
 }
 
 static bool enter_notify_event(GtkWidget *w, GdkEventCrossing *event)
@@ -721,8 +762,7 @@ static bool enter_notify_event(GtkWidget *w, GdkEventCrossing *event)
 static bool leave_notify_event(GtkWidget *w, GdkEventCrossing *event)
 {
 	ValaPanelToplevel *self = (ValaPanelToplevel *)w;
-	if (self->autohide &&
-	    (event->detail != GDK_NOTIFY_INFERIOR && event->detail != GDK_NOTIFY_VIRTUAL))
+	if (self->autohide && (event->detail != GDK_NOTIFY_INFERIOR))
 		ah_hide(self);
 	return false;
 }
@@ -732,7 +772,10 @@ static void grab_notify(ValaPanelToplevel *self, bool was_grabbed)
 	if (!was_grabbed)
 		self->ah_state = AH_GRAB;
 	else if (self->autohide)
+	{
+		self->ah_state = AH_VISIBLE;
 		ah_hide(self);
+	}
 }
 
 /*
@@ -977,6 +1020,7 @@ void vala_panel_toplevel_init(ValaPanelToplevel *self)
 	self->font            = g_strdup("");
 	self->background_file = g_strdup("");
 	self->context_menu    = NULL;
+	self->ah_state        = AH_VISIBLE; // We starts as Visible to init autohide chain properly
 }
 
 void vala_panel_toplevel_class_init(ValaPanelToplevelClass *parent)
